#!/usr/bin/perl -w

# Linux Application Checker
# Application Analysis Data Generator (analysis.pl)
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use strict;
use FindBin;
use File::Basename;

sub BEGIN { push @INC, $FindBin::Bin; }

use Analysis;
#use Print_ref; # DBG:

# dummy # DBG:
sub elapsed {
    return "";
}

my ($lsb_version, $host_platform, $updata_file) = @ARGV;

my $res_dir = dirname($updata_file);
$res_dir = "." if $res_dir eq "";

my $data_dir = $FindBin::Bin;

my $arch_f = lc($host_platform);

## Load and analyze application data ##

# Load distribution list (contains distribution name, version, database ID, architecture) 
Analysis::load_distrs($data_dir."/Distributions_".$arch_f)
    or return "Failed to read the Distributions file.";
print elapsed("load_distrs");

Analysis::read_upload_data($updata_file, $host_platform);
print elapsed("read_upload_data");

Analysis::read_LibIndex( $data_dir."/LibIndex_".$arch_f );
print elapsed("read_LibIndex");

Analysis::read_UnapprovedLibs( $data_dir."/UnapprovedLibs_".$arch_f );
print elapsed("UnapprovedLibs");

Analysis::link_libs($Analysis::reqlibs);
print elapsed("link_libs");

Analysis::read_LSB_Libs($data_dir."/LSB_Libs_".$arch_f);
print elapsed("read_LSB_Libs");

Analysis::read_LibAttrs($data_dir."/LSB_LibAttrs_".$arch_f, $Analysis::reqlibs);
print elapsed("read_LibAttrs");

Analysis::read_libKB($data_dir."/LibKB_".$arch_f);
print elapsed("read_libKB");

# scan local libs
Analysis::get_unknown_libs($host_platform);
print elapsed("scan local libs");

my $int_index = Analysis::read_IIndex($data_dir."/IIndex_".$arch_f);
print elapsed("read_IIndex");

Analysis::find_interfaces($data_dir."/Interfaces_".$arch_f, $int_index);
$int_index = undef; # No more needed
print elapsed("find_interfaces");

Analysis::find_libifaces( $data_dir."/LibIfaces_".$arch_f);
print elapsed("find_libifaces");

Analysis::link_ints();
print elapsed("link_ints");

Analysis::read_LSB_Ints($data_dir."/LSB_Ints_".$arch_f);
print elapsed("read_LSB_Ints");

Analysis::read_IntAttrs($data_dir."/LSB_IntAttrs_".$arch_f, $Analysis::reqints);
print elapsed("read_IntAttrs");

Analysis::unmangle();
print elapsed("unmangle");

# TODO:
#Analysis::read_intKB($data_dir."/IntKB");
#print elapsed("read_libKB");

## Save analysis results ##

Analysis::lib_dump( $res_dir."/libs", $res_dir."/lib_attrs", $res_dir."/lib_kb" );
print elapsed("lib_dump");

Analysis::int_dump( $res_dir."/ints", $res_dir."/int_attrs", $res_dir."/int_kb" );
print elapsed("int_dump");

Analysis::save_components( $res_dir."/components" );
print elapsed("components_dump");

system("cp '$data_dir/Distributions_".$arch_f."'  '$res_dir/Distributions'");

# ---------------------------

Analysis::read_lib_dump( $res_dir."/libs" );
print elapsed("read_lib_dump");

#Analysis::read_LibAttrs($res_dir."/lib_attrs", $Analysis::libs);
#print elapsed("read_LibAttrs");

Analysis::read_int_dump( $res_dir."/ints" );
print elapsed("read_int_dump");

#Analysis::read_IntAttrs($res_dir."/int_attrs", $Analysis::ints);
#print elapsed("read_IntAttrs");

# ---------------------------

my $total_verdict = '';
foreach my $Did (keys %$Analysis::all_distrs) {
	my ($status) = Analysis::distr_status($Did);
	if ($status eq 'MISSED') {
		if ($total_verdict eq '') {
			$total_verdict = 'Failed';
		}
		elsif ($total_verdict ne 'Failed') {
			$total_verdict = 'Warnings';
			last;
		}
	}
	elsif ($status eq 'OK') {
		if ($total_verdict eq '') {
			$total_verdict = 'Success';
		}
		elsif ($total_verdict ne 'Success') {
			$total_verdict = 'Warnings';
			last;
		}
	}
	else {
		$total_verdict = 'Warnings';
		last;
	}
}

if (open(INFO, ">$res_dir/INFO")) {
	my $app_name = basename($updata_file);
	$app_name =~ s/^(.*)_\Q${host_platform}\E_upload_data/$1/;
	my $res_dir_esc = $res_dir;
	$res_dir_esc =~ s/ /\\ /g;
	my $comment = `cat $res_dir_esc/test_logs/test_comments`;
	print INFO <<DATA;
Application: $app_name
Verdict: $total_verdict

$comment
DATA
	close(INFO);
}
else {
	print STDERR "Failed to open INFO file for writing:\n$!\n";
}


exit 0;
